#COMPILE EXE
DEFDBL a - z        'Default type is DOUBLE
GLOBAL pi, dr  AS DOUBLE

FUNCTION ATAN2(x,y)
    mag =  SQR(x^2 + y^2)
    IF (mag + x) > 0.00000001 THEN ATAN2 = 2 * ATN(y / (mag + x)) ELSE ATAN2 = -pi
END FUNCTION

FUNCTION ACOS(X) : ACOS = ATN((SQR(1-X*X))/X) + pi/2*(1-SGN(X)) : END FUNCTION

FUNCTION PBMAIN () AS LONG
  pi =  3.141592653589793236462643383279
  dr = 180 / pi
COLOR 14
PRINT "Demonstration of WGS84 to Airy (GBR36)  conversion "
DO
    PRINT
    COLOR 7
    INPUT "Lat , Long  (in either mapping)  ",LatIn, LonIn
    CALL WGS84ToAiry(LatIn,LonIn, LatA, LonA)
    CALL DistBear(LatIn, LonIn, LatA, LonA, D1, B1, BB1)
    COLOR 3
    PRINT USING$ ("To Airy    ###.#######   ###.#######   Distance ####.#m Bearing ###",LatA, LonA, D1* 1000, B1)
    CALL AiryToWGS84(LatIn, LonIn, LatW, LonW)
    CALL DistBear(LatIn, LonIn, LatW, LonW, D2, B2, BB2)
    PRINT USING$ ("To WGS84   ###.#######   ###.#######            ####.#m         ###",LatW, LonW,D2 * 1000,B2)
LOOP  UNTIL ABS(Latin) < .0000000001 AND ABS (Lonin) < .0000000001


END FUNCTION

 '.................................................................................
SUB AiryToWGS84(Latin, Lonin, Lat2, Long2)
' Converts Airy Geoid Lat/Long to WGS84 Lat/Long
' Lat and Long are inputs
' From the paper "The Geoid, James Myatt 7 July 2004"
' Lat and Long are inputs

        AiryMajoraxis =         6377563.396            ' Airy Major Axis
        AiryMinoraxis =         6356256.910            ' Airy Minor Axis
        Airyesq =               0.0066705397616        '(AiryMajoraxis * AiryMajoraxis - AiryMinoraxis * AiryMinoraxis) / (AiryMajoraxis * AiryMajoraxis);
        WGS84Majoraxis = 6378137.000  ' WGS84 Major Axis
        WGS84Minoraxis = 6356752.3141 ' WGS84 Minor Axis
        WGS84esq = 0.0066943800355    ' (WGS84Majoraxis * WGS84Majoraxis - WGS84Minoraxis * WGS84Minoraxis) / (WGS84Majoraxis * WGS84Majoraxis);
' 7 parameter Helmert Transform coefficients OSGB - WGS84 values from from James Myatt paper
         tx = 446.448
         ty = -125.157
         tz = 542.060
         s = -20.4894e-6
         rx = 0.1502 / dr / 3600.0
         ry = 0.2470 / dr / 3600.0
         rz = 0.8421 / dr / 3600.0
         h = 0.0

        ' convert to radians
        lat1 = Latin / dr
        lon1 = Lonin / dr

        ' convert OSGB lat/lon to X,Y,Z
        v = AiryMajoraxis / SQR(1.0 - Airyesq * SIN(lat1) * SIN(lat1))     ' eqn 2.1
        x1 = (v + h) * COS(lat1) * COS(lon1)                                ' eqn 2.2
        y1 = (v + h) * COS(lat1) * SIN(lon1)                                ' eqn 2.3
        z1 = ((1.0 - Airyesq) * v + h) * SIN(lat1)                          ' eqn 2.4

        ' rotate and translate to new coordinate system
        ' with Helmert Transformation coefficients, eqn 3.1

        x2 = tx + (1.0 + s) * x1 - rz        * y1 + ry       * z1
        y2 = ty + rz        * x1 + (1.0 + s) * y1 - rx       * z1
        z2 = tz - ry        * x1 + rx        * y1 +(1.0 + s) * z1

        ' convert X,Y,Z to WGS84 lat/lon

        lon2 = ATAN2(x2, y2)

        p = SQR(x2 * x2 + y2 * y2)
        lat2 = ATAN2 (p * (1.0 - WGS84esq) , z2)

        DO
                lat = lat2
                v = WGS84Majoraxis / SQR(1.0 - WGS84esq * SIN(lat) * SIN(lat))
                lat2 = ATAN2(p, z2 + WGS84esq * v * SIN(lat))
        LOOP WHILE (ABS(lat - lat2) > 1.0e-8)
              Lat2 = lat2 * dr
              Long2 = lon2 * dr
END SUB
  '.................................................................................
SUB WGS84ToAiry(Latin,Lonin, Lat2, Long2)

' Converts WGS84 Lat/Long to Airy Geoid Lat/Long
' From the paper "The Geoid, James Myatt 7 July 2004"
' Lat and Long are inputs

        AiryMajoraxis =         6377563.396            ' Airy Major Axis
        AiryMinoraxis =         6356256.910            ' Airy Minor Axis
        Airyesq =               0.0066705397616        '(AiryMajoraxis * AiryMajoraxis - AiryMinoraxis * AiryMinoraxis) / (AiryMajoraxis * AiryMajoraxis);
        WGS84Majoraxis = 6378137.000  ' WGS84 Major Axis
        WGS84Minoraxis = 6356752.3141 ' WGS84 Minor Axis
        WGS84esq = 0.0066943800355    ' (WGS84Majoraxis * WGS84Majoraxis - WGS84Minoraxis * WGS84Minoraxis) / (WGS84Majoraxis * WGS84Majoraxis);

        ' 7 parameter Helmert Transform coefficients WGS84 - OSGB
         s = 20.4894e-6 ' scale factor
         tx = -446.448    ' x translation
         ty = 125.157   ' y translation
         tz = -542.060    ' z translation
         rx = -0.1502 / dr / 3600.0  ' x rotation
         ry = -0.2470 / dr / 3600.0  ' y rotation
         rz = -0.8421 / dr / 3600.0  ' z rotation
         h = 0.0 ' Ellipsoidal height (we will set this to zero)

        ' convert to radians
        lat1 = Latin / dr
        lon1 = Lonin / dr

        ' convert WGS84 lat/lon to X,Y,Z

        v = WGS84Majoraxis / SQR(1.0 - WGS84esq * SIN(lat1) * SIN(lat1))  ' eqn 2.1

        x1 = (v + h) * COS(lat1) * COS(lon1)                               ' eqn 2.2
        y1 = (v + h) * COS(lat1) * SIN(lon1)                               ' eqn 2.3
        z1 = ((1.0 - WGS84esq) * v + h) * SIN(lat1)                        ' eqn 2.4

        ' rotate and translate to new coordinate system
        ' with Helmert Transformation coefficients, eqn 3.1

        x2 = tx + (1.0 + s) * x1 - rz       * y1 + ry       * z1
        y2 = ty + rz       * x1 + (1.0 + s) * y1 - rx       * z1
        z2 = tz - ry       * x1 + rx        * y1 + (1.0 + s)* z1

        ' convert X,Y,Z back to lat/lon


        lon2 = ATAN2(x2, y2) ' eqn 2.5
        ' Latitude needs iteration

        p = SQR(x2 * x2 + y2 * y2)
        lat2 = ATAN2(p * (1.0 - WGS84esq), z2)


        DO
                lat = lat2
                v = AiryMajoraxis / SQR(1.0 - Airyesq * SIN(lat)* SIN(lat))
                lat2 = ATAN2(p,z2 + Airyesq * v * SIN(lat))

        LOOP WHILE (ABS(lat - lat2) > 1.0e8)
        Lat2 = lat2 * dr
        Long2 = lon2 * dr
 END SUB
  '.................................................................................
  '.............................................................................................
SUB DistBear(BaseLat, BaseLon, Lat, Lon, Distance, Bearing, BackBearing)
    re = 6371.29
    BaseLatR = BaseLat / dr :  BaseLonR = BaseLon / dr
    LatR = Lat / dr :  LonR = Lon / dr

    CSarc = SIN(BaseLatR) * SIN(LatR) + COS(BaseLatR) * COS(LatR) * COS(LonR - BaseLonR)
    IF CSarc >1 THEN CSarc = 1
    Angle = ACOS(CSarc)
    Distance = ABS(re * Angle)
    CSbear = (SIN(LatR) - SIN(BaseLatR) * CSarc) / (COS(BaseLatR) * SIN(Angle))
    BackCSbear =  (SIN(BaseLatR) - SIN(LatR) * CSarc) / (COS(LatR) * SIN(Angle))
    IF ABS(CSBear) > 1 THEN CSBear = 1
    IF ABS(BackCsBear) > 1 THEN BackCsBear = 1
    Bearing = dr * ACOS(CSbear)
    BackBearing = dr * ACOS(BackCsBear)

    IF Lon < BaseLon THEN Bearing = 360 - Bearing
    IF BaseLon < Lon THEN BackBearing = 360 - BackBearing
    IF Bearing > 360 THEN Bearing = Bearing - 360
    IF Bearing < 0 THEN Bearing = Bearing + 360
    IF BackBearing > 360 THEN BackBearing = BackBearing - 360
    IF BackBearing < 0 THEN BackBearing = BackBearing + 360
END SUB
